* This is an example of assembler source code which is assembled by the
* METACOMCO assembler or similar into a binary code file, which is loaded
* by the SUPERFORTH word LOAD_BIN into the SUPERFORTH dictionary.
* The format in the examples below must be followed to ensure that the
* correct number of bytes at the head of each word is present. This
* header is :
*
*           $4AFB          a marker ( word )
*           name_length    number of characters ( byte )
*           name           in ASCII
*           ( padding )    0 or 1 byte, only if length of name is even, to
*                          ensure the following code is on a word boundary.
*
* This header is created using the "code" macro which must be used in the
* form
*              code     'name',label
*               .          .              assembler instructions
*               .          .
*              next                       or a branch to a next
* label
*           the label is essential so that LOAD_BIN knows how much code to
*           load into a parameter field.
*
* Note that position independent code must be generated, you can refer to
* labels in other code definitions since the number of bytes in the
* above header is exactly the same as in the SUPERFORTH header itself.
* Each code word must end in a "next" macro or a branch to a "next" macro
* See chapter 11 of the SUPERFORTH manual.
*
* A 0 must be present at the end of the file to mark the end of the file,
* this is generated below by the "end_file" macro.
*
* Assemblers other than Metacomco's can be used in a similar way but the
* syntax for the macros will be different.
*
*
*
marker         equ      $4AFB             To mark the start of a code word
*
* macros
*
next           macro                      Fetches and executes the next
               move.w   (a1)+,d1          SUPERFORTH word, must be executed
               movea.w  0(a2,d1.w),a5     at the end of a code word
               jmp      0(a2,a5.w)
               endm
*
*
code           macro                      Creates a header
               dc.w     marker            Marks the start of a word
               dc.b     strg_end\@-strg_start\@     Gives the string length
strg_start\@   dc.b     \1                The primitive's name
strg_end\@     equ      *
               cnop     0,2               Pads to an even address
               dc.w     \2-*-2            Gives the code length
               endm
*
end_file       macro
               dc.w     0                 Must be placed at end of the code
               endm
*
*
******************************************
* three example machine code definitions *
******************************************
*
               code     'NOR',nor_end     Logical nor of top 2 words on stack
               or.w     (a3)+,d2
               not.w    d2
               bra.s    done
nor_end
*
               code     '3*',times_3_end  Multiplies top of stack by 3
               move.w   d2,d1             d2 holds top of stack
               add.w    d1,d2             Crude but quick!
               add.w    d1,d2
done
               next                       Note, must not label a macro call
times_3_end
*
               code     '3DUP',three_dup_end  Duplicates top 3 words on stack
               move.w   d2,-(a3)          Push top of stack
               move.w   4(a3),-(a3)
               move.w   4(a3),-(a3)       d2 is ok
               bra.s    done
three_dup_end
*
               end_file
*
               end
